@@ -9,6 +9,7 @@ Endpoints for server management:
 - POST /api/servers/{name}/setup - Setup server (async job)
 - POST /api/servers/{name}/dns - Configure DNS for server
 - GET /api/servers/{name}/dns - Get DNS configuration
+- POST /api/servers/{name}/exec - Execute remote SSH command
 """
 
 from fastapi import APIRouter, Depends, HTTPException, status
@@ -30,11 +31,16 @@ try:
         DNSGetResponse,
         DNSConfig
     )
+    from ..models.remote_exec import (
+        RemoteExecRequest,
+        RemoteExecResponse,
+        RemoteExecErrorResponse
+    )
     from ...job_manager import JobManager
     from ...orchestrator import Orchestrator
 except ImportError:
-    from src.api.dependencies import get_job_manager, get_orchestrator
-    from src.api.models.server import (
+    from api.dependencies import get_job_manager, get_orchestrator
+    from api.models.server import (
         ServerCreateRequest,
         ServerSetupRequest,
         ServerInfo,
@@ -47,8 +53,13 @@ except ImportError:
         DNSGetResponse,
         DNSConfig
     )
-    from src.job_manager import JobManager
-    from src.orchestrator import Orchestrator
+    from api.models.remote_exec import (
+        RemoteExecRequest,
+        RemoteExecResponse,
+        RemoteExecErrorResponse
+    )
+    from job_manager import JobManager
+    from orchestrator import Orchestrator
 
 logger = logging.getLogger(__name__)
 
@@ -130,31 +141,21 @@ def _sync_servers_with_provider(orchestrator: Orchestrator) -> None:
     3. Marks servers in state but not in provider as 'deleted_externally'
     4. Updates status/IP for servers present in both
 
-    Gracefully degrades if provider unavailable (no token, network error, etc.)
+    Raises ValueError if no provider is configured (missing hetzner_token).
+    Gracefully degrades on network errors, logging warning instead of failing.
     """
     try:
-        # Try to initialize provider (lazy load from vault)
-        # Detect provider from available tokens
-        if not orchestrator.provider:
-            # Try Hetzner first (primary provider)
-            token = orchestrator.storage.secrets.get_secret("hetzner_token")
+        # Get provider (auto-initializes from vault if needed)
+        provider = orchestrator.provider_manager.get_provider()
 
-            if token:
-                from src.providers.hetzner import HetznerProvider
-                orchestrator.provider = HetznerProvider(token)
-                logger.debug("Hetzner provider initialized for sync")
-            else:
-                # Try DigitalOcean as fallback
-                token = orchestrator.storage.secrets.get_secret("digitalocean_token")
-                if token:
-                    logger.warning("DigitalOcean provider not yet implemented - skipping sync")
-                    return
-                else:
-                    logger.debug("No provider token available - skipping sync")
-                    return
+        if not provider:
+            raise ValueError(
+                "No cloud provider configured. Hetzner API token (hetzner_token) is required to list servers. "
+                "Please configure it using the manage-secrets tool."
+            )
 
         # Fetch servers from provider
-        provider_servers = orchestrator.provider.list_servers()
+        provider_servers = provider.list_servers()
         logger.info(f"Found {len(provider_servers)} servers in provider")
 
         # Get current state
@@ -289,21 +290,18 @@ async def get_server(
         server_id = server_data.get("id")
 
         if server_id:
-            # Initialize provider if needed (lazy load from vault)
-            # Detect provider from available tokens
-            if not orchestrator.provider:
-                # Try Hetzner first (primary provider)
-                token = orchestrator.storage.secrets.get_secret("hetzner_token")
-                if token:
-                    from src.providers.hetzner import HetznerProvider
-                    orchestrator.provider = HetznerProvider(token)
-                    logger.debug("Hetzner provider initialized for verification")
+            # Get provider from provider_manager (auto-initializes from vault if needed)
+            try:
+                provider = orchestrator.provider_manager.get_provider()
+            except Exception as e:
+                logger.warning(f"Could not initialize provider for verification: {e}")
+                provider = None
 
             # Only verify if provider was successfully initialized
-            if orchestrator.provider:
+            if provider:
                 try:
                     # Try to get server from Hetzner
-                    provider_server = orchestrator.provider.get_server(server_id)
+                    provider_server = provider.get_server(server_id)
 
                     # Update state with fresh data from provider
                     if provider_server:
@@ -665,3 +663,172 @@ async def get_server_dns(
     except Exception as e:
         logger.error(f"Failed to get DNS for {name}: {e}", exc_info=True)
         raise HTTPException(status_code=500, detail=str(e))
+
+
+@router.post("/{name}/exec")
+async def execute_remote_command(
+    name: str,
+    request: RemoteExecRequest,
+    job_manager: JobManager = Depends(get_job_manager),
+    orchestrator: Orchestrator = Depends(get_orchestrator)
+):
+    """
+    Execute a command on a remote server via SSH (sync or async)
+
+    Executes a shell command on the specified server using AsyncSSH.
+    Can run synchronously (immediate result) or asynchronously via job system.
+
+    Execution Mode Decision:
+    - IF use_job=True OR timeout > 60s → Async (job system) → 202 Accepted
+    - ELSE → Sync (immediate execution) → 200 OK
+
+    Security:
+    - Commands are validated against dangerous patterns (rm -rf /, dd, mkfs, etc.)
+    - Output is truncated to 10KB to prevent memory issues
+    - Commands timeout after specified duration (max 300s)
+
+    Use Cases:
+    - Quick diagnostics (docker ps, df -h, systemctl status) → sync
+    - Log inspection (tail -n 100 /var/log/app.log) → sync
+    - Long-running commands (docker logs with large output) → async via use_job=True
+
+    Args:
+        name: Server name
+        request: Command execution request (command, timeout, working_dir, use_job)
+
+    Returns:
+        - 200 OK: RemoteExecResponse (sync execution)
+        - 202 Accepted: Job response with job_id (async execution)
+
+    Raises:
+        404: Server not found or SSH key missing
+        400: Dangerous command rejected or validation failed
+        500: SSH connection or execution error
+
+    Example (sync):
+        POST /api/servers/prod-server/exec
+        {
+            "command": "docker ps",
+            "timeout": 10
+        }
+
+    Example (async):
+        POST /api/servers/prod-server/exec
+        {
+            "command": "docker logs container-id --tail 10000",
+            "timeout": 120,
+            "use_job": true
+        }
+    """
+    # Check if server exists
+    server_data = orchestrator.storage.state.get_server(name)
+    if not server_data:
+        raise HTTPException(
+            status_code=404,
+            detail=f"Server {name} not found"
+        )
+
+    # Decision: Job-based (async) vs Direct execution (sync)
+    # Job mode if: use_job=True OR timeout > 60s
+    use_job_mode = request.use_job or request.timeout > 60
+
+    if use_job_mode:
+        # ASYNC PATH: Create job and return 202 Accepted
+        try:
+            job = await job_manager.create_job(
+                job_type="remote_exec",
+                params={
+                    "server_name": name,
+                    "command": request.command,
+                    "timeout": request.timeout,
+                    "working_dir": request.working_dir
+                }
+            )
+
+            logger.info(
+                f"Created job {job.job_id} for remote command execution on {name}: "
+                f"command='{request.command}', timeout={request.timeout}s, "
+                f"use_job={request.use_job}"
+            )
+
+            from fastapi.responses import JSONResponse
+            return JSONResponse(
+                status_code=status.HTTP_202_ACCEPTED,
+                content={
+                    "job_id": job.job_id,
+                    "message": f"Command execution started via job system on {name}",
+                    "server_name": name,
+                    "command": request.command,
+                    "timeout": request.timeout
+                }
+            )
+
+        except Exception as e:
+            logger.error(f"Failed to create remote_exec job for {name}: {e}", exc_info=True)
+            raise HTTPException(status_code=500, detail=f"Failed to create job: {str(e)}")
+
+    # SYNC PATH: Execute command directly
+    try:
+        # Execute command via orchestrator
+        result = await orchestrator.execute_remote_command(
+            server_name=name,
+            command=request.command,
+            timeout=request.timeout,
+            working_dir=request.working_dir
+        )
+
+        logger.info(
+            f"Remote command executed on {name}: "
+            f"exit_code={result['exit_code']}, "
+            f"stdout_len={len(result['stdout'])}, "
+            f"stderr_len={len(result['stderr'])}"
+        )
+
+        # Build response
+        return RemoteExecResponse(
+            success=result["success"],
+            server_name=name,
+            command=request.command,
+            stdout=result["stdout"],
+            stderr=result["stderr"],
+            exit_code=result["exit_code"],
+            timeout_seconds=request.timeout,
+            working_dir=request.working_dir
+        )
+
+    except ValueError as e:
+        # Validation errors (dangerous command, empty command, etc.)
+        error_msg = str(e)
+        logger.warning(f"Command validation failed for {name}: {error_msg}")
+
+        raise HTTPException(
+            status_code=400,
+            detail=error_msg
+        )
+
+    except FileNotFoundError as e:
+        # SSH key not found
+        logger.error(f"SSH key not found for {name}: {e}")
+
+        raise HTTPException(
+            status_code=404,
+            detail=f"SSH key not found for server {name}"
+        )
+
+    except TimeoutError:
+        # Command timed out
+        logger.error(f"Command timed out on {name} after {request.timeout}s")
+
+        raise HTTPException(
+            status_code=500,
+            detail=f"Command timed out after {request.timeout} seconds"
+        )
+
+    except Exception as e:
+        # SSH connection or execution errors
+        logger.error(f"Remote command execution failed on {name}: {e}", exc_info=True)
+
+        raise HTTPException(
+            status_code=500,
+            detail=f"Command execution failed: {str(e)}"
+        )
